@@ -62,6 +62,9 @@
 #ifdef HAVE_SYS_ELF_386_H
 #include <sys/elf_386.h>
 #endif
+#ifdef HAVE_SYS_ELF_AMD64_H
+#include <sys/elf_amd64.h>
+#endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -161,95 +164,48 @@ void ELFExport::ScanConstant(byte *addr, ScanRelocationKind code)
     {
     case PROCESS_RELOC_DIRECT: // 32 or 64 bit address of target
         {
-            ElfXX_Rel reloc;
-            setRelocationAddress(addr, &reloc.r_offset);
-            reloc.r_info = ELFXX_R_INFO(AreaToSym(aArea), directReloc);
+            PolyWord r = createRelocation(p, addr);
+            POLYUNSIGNED w = r.AsUnsigned();
             for (unsigned i = 0; i < sizeof(PolyWord); i++)
             {
-                addr[i] = (byte)(offset & 0xff);
-                offset >>= 8;
+                addr[i] = (byte)(w & 0xff);
+                w >>= 8;
             }
-            fwrite(&reloc, sizeof(reloc), 1, exportFile);
-            relocationCount++;
         }
         break;
 #if(defined(HOSTARCHITECTURE_X86) || defined(HOSTARCHITECTURE_X86_64))
      case PROCESS_RELOC_I386RELATIVE:         // 32 bit relative address
         {
-            ElfXX_Rel reloc;
-            setRelocationAddress(addr, &reloc.r_offset);
-             // We seem to need to subtract 4 bytes to get the correct offset in ELF
-            offset -= 4;
-            reloc.r_info = ELFXX_R_INFO(AreaToSym(aArea), R_386_PC32);
-            for (unsigned i = 0; i < 4; i++)
+            if (useRela)
             {
-                addr[i] = (byte)(offset & 0xff);
-                offset >>= 8;
+                ElfXX_Rela reloc;
+                setRelocationAddress(addr, &reloc.r_offset);
+                // We seem to need to subtract 4 bytes to get the correct offset in ELF
+                offset -= 4;
+                reloc.r_info = ELFXX_R_INFO(AreaToSym(aArea), R_386_PC32);
+                reloc.r_addend = offset;
+                // Clear the field.  Even though it's not supposed to be used with Rela the
+                // Linux linker at least seems to add the value in here sometimes.
+                memset(addr, 0, 4);
+                fwrite(&reloc, sizeof(reloc), 1, exportFile);
+            }
+            else
+            {
+                ElfXX_Rel reloc;
+                setRelocationAddress(addr, &reloc.r_offset);
+                 // We seem to need to subtract 4 bytes to get the correct offset in ELF
+                offset -= 4;
+                reloc.r_info = ELFXX_R_INFO(AreaToSym(aArea), R_386_PC32);
+                for (unsigned i = 0; i < 4; i++)
+                {
+                    addr[i] = (byte)(offset & 0xff);
+                    offset >>= 8;
+                }
+                fwrite(&reloc, sizeof(reloc), 1, exportFile);
             }
-            fwrite(&reloc, sizeof(reloc), 1, exportFile);
             relocationCount++;
         }
         break;
-#endif
-#ifdef HOSTARCHITECTURE_PPC
-    case PROCESS_RELOC_PPCDUAL16SIGNED:       // Power PC - two consecutive words
-    case PROCESS_RELOC_PPCDUAL16UNSIGNED:
-        {
-            ElfXX_Rela reloc;
-            setRelocationAddress(addr+2 /* actual bytes to be updated */, &reloc.r_offset);
-            // We need two relocations here.
-            reloc.r_info = ELFXX_R_INFO(AreaToSym(aArea),
-                    code == PROCESS_RELOC_PPCDUAL16SIGNED ? R_PPC_ADDR16_HA : R_PPC_ADDR16_HI);
-            reloc.r_addend = offset;
-            fwrite(&reloc, sizeof(reloc), 1, exportFile);
-            relocationCount++;
-
-            setRelocationAddress(addr+sizeof(PolyWord)+2, &reloc.r_offset);
-            reloc.r_info = ELFXX_R_INFO(AreaToSym(aArea), R_PPC_ADDR16_LO);
-            reloc.r_addend = offset;
-            fwrite(&reloc, sizeof(reloc), 1, exportFile);
-            relocationCount++;
-            // Set the constant values to zero.  This doesn't seem to be necessary
-            // on the PPC but can't do any harm.
-            POLYUNSIGNED *caddr = (POLYUNSIGNED *)addr;
-            caddr[0] = caddr[0] & 0xffff0000;
-            caddr[1] = caddr[1] & 0xffff0000;
-        }
-        break;
-#endif
-#ifdef HOSTARCHITECTURE_SPARC
-    case PROCESS_RELOC_SPARCDUAL: // Sparc - two consecutive words
-        {
-            ElfXX_Rela reloc;
-            setRelocationAddress(addr, &reloc.r_offset);
-            // We need two relocations here.
-            reloc.r_info = ELFXX_R_INFO(AreaToSym(aArea), R_SPARC_HI22);
-            reloc.r_addend = offset;
-            fwrite(&reloc, sizeof(reloc), 1, exportFile);
-            relocationCount++;
-
-            setRelocationAddress(addr+sizeof(PolyWord), &reloc.r_offset);
-            reloc.r_info = ELFXX_R_INFO(AreaToSym(aArea), R_SPARC_LO10);
-            reloc.r_addend = offset;
-            fwrite(&reloc, sizeof(reloc), 1, exportFile);
-            relocationCount++;
-            POLYUNSIGNED *caddr = (POLYUNSIGNED *)addr;
-            caddr[0] = caddr[0] & 0xffc00000;
-            caddr[1] = caddr[1] & 0xfffff000;
-        }
-        break;
-    case PROCESS_RELOC_SPARCRELATIVE: // Sparc 30-bit relative address
-        {
-            ElfXX_Rela reloc;
-            setRelocationAddress(addr, &reloc.r_offset);
-            reloc.r_info = ELFXX_R_INFO(AreaToSym(aArea), R_SPARC_WDISP30);
-            reloc.r_addend = offset;
-            fwrite(&reloc, sizeof(reloc), 1, exportFile);
-            relocationCount++;
-            POLYUNSIGNED *caddr = (POLYUNSIGNED *)addr;
-            caddr[0] = caddr[0] & 0xc0000000;
-        }
-        break;
 #endif
         default:
             ASSERT(0); // Wrong type of relocation for this architecture.
@@ -361,9 +317,11 @@ void ELFExport::exportStore(void)
        relocations) as well as the addend. To be safe, whenever we use an ELF32_Rela
        relocation we always zero the location to be relocated. */
 #elif defined(HOSTARCHITECTURE_X86_64)
+    /* It seems Solaris/X86-64 only supports ELF64_Rela relocations.  It appears that
+       Linux will support either so we now use Rela on X86-64. */
     fhdr.e_machine = EM_X86_64;
     directReloc = R_X86_64_64;
-    useRela = false;
+    useRela = true;
 #elif defined(HOSTARCHITECTURE_ARM)
 #ifndef EF_ARM_EABI_VER4 
 #define EF_ARM_EABI_VER4     0x04000000
@@ -553,9 +511,11 @@ void ELFExport::exportStore(void)
                 p++;
                 PolyObject *obj = (PolyObject*)p;
                 POLYUNSIGNED length = obj->Length();
-                relocateObject(obj);
+                // Update any constants before processing the object
+                // We need that for relative jumps/calls in X86/64.
                 if (length != 0 && obj->IsCodeObject())
                     machineDependent->ScanConstantsWithinCode(obj, this);
+                relocateObject(obj);
                 p += length;
             }
         }
@@ -607,7 +567,6 @@ void ELFExport::exportStore(void)
 
     exportDescription exports;
     memset(&exports, 0, sizeof(exports));
-    memset(memTable, 0, sizeof(memTable));
     exports.structLength = sizeof(exportDescription);
     exports.memTableSize = sizeof(memoryTableEntry);
     exports.memTableEntries = memTableEntries;
