@@ -95,26 +95,27 @@ const PolyWord Zero = TAGGED(0);
    CHECKED_REGS + \
    UNCHECKED_REGS + \
    EXTRA_STACK)
-/*
+
 class IntTaskData: public MDTaskData {
 public:
-    IntTaskData(): allocWords(0) {}
-    POLYUNSIGNED allocWords; // The words to allocate.
-};*/
+    IntTaskData(): interrupt_requested(false) {}
+    
+    bool interrupt_requested;
+};
 
 // Special values for return addresses or in the address of an exception handler.
 // In an exception handler SPECIAL_PC_TRACE_EX means trace this exception, as
 // a return address it means exception_trace has returned.
 #define SPECIAL_PC_TRACE_EX    TAGGED(0)
 #define SPECIAL_PC_END_THREAD  TAGGED(1)
-
+#define SPECIAL_PC_TRACE_EX_FN TAGGED(2)
 
 class Interpreter : public MachineDependent {
 public:
     Interpreter() {}
 
     // Create a task data object.
-    virtual MDTaskData *CreateTaskData(void) { return new MDTaskData(); }
+    virtual MDTaskData *CreateTaskData(void) { return new IntTaskData(); }
 
     virtual void InitStackFrame(TaskData *taskData, StackSpace *space, Handle proc, Handle arg);
     // Switch to Poly and return with the io function to call.
@@ -141,8 +142,14 @@ public:
     // calling are actually IO functions and don't have code addresses.  Actually
     // we could instead make the interpreter handle small tagged integers as
     // code addresses specially.
-    virtual void SetExceptionTrace(TaskData *taskData) { ASSERT(false); }
+    virtual void SetExceptionTrace(TaskData *taskData, bool isLegacy) { ASSERT(false); }
     virtual void CallCodeTupled(TaskData *taskData) { ASSERT(false); }
+    // Increment or decrement the first word of the object pointed to by the
+    // mutex argument and return the new value.
+    virtual Handle AtomicIncrement(TaskData *taskData, Handle mutexp);
+    virtual Handle AtomicDecrement(TaskData *taskData, Handle mutexp);
+    // Set a mutex to one.
+    virtual void AtomicReset(TaskData *taskData, Handle mutexp);
 };
 
 void Interpreter::InitStackFrame(TaskData *taskData, StackSpace *space, Handle proc, Handle arg)
@@ -175,14 +182,12 @@ void Interpreter::InitStackFrame(TaskData *taskData, StackSpace *space, Handle p
     *(--stack->p_sp) = closure; /* Closure address */
 }
 
-int interrupt_requested = 0;
-int profile_count_wanted = 0;
-
 void Interpreter::InterruptCode(TaskData *taskData)
 /* Stop the Poly code at a suitable place. */
 /* We may get an asynchronous interrupt at any time. */
 {
-    interrupt_requested = 1;
+    IntTaskData *itd = (IntTaskData *)taskData->mdTaskData;
+    itd->interrupt_requested = true;
 }
 
 
@@ -225,6 +230,7 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
     pc = taskData->stack->stack()->p_pc;
     li = (unsigned)UNTAGGED(taskData->stack->stack()->p_reg[1]);
     sl = (PolyWord*)taskData->stack->stack()+OVERFLOW_STACK_SIZE;
+    IntTaskData *itd = (IntTaskData *)taskData->mdTaskData;
 
     if (li != 256) goto RETRY; /* Re-execute instruction if necessary. */
 
@@ -252,19 +258,14 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
             goto RESTART;
         }
 
-        if (interrupt_requested) {
-            interrupt_requested = 0;
+        if (itd->interrupt_requested) {
+            itd->interrupt_requested = false;
             taskData->stack->stack()->p_sp = sp;
             taskData->stack->stack()->p_pc = pc;
             taskData->stack->stack()->p_reg[1] = TAGGED(li);
             return -1;
         }
 
-        if (profile_count_wanted) {
-            add_count(taskData, pc, sp, profile_count_wanted);
-            profile_count_wanted = 0;
-        }
-
         switch(li) {
 
         case INSTR_enter_int: pc++; /* Skip the argument. */ break;
@@ -282,7 +283,14 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
             *(--sp) = PolyWord::FromStackAddr(taskData->stack->stack()->p_hr); /* Push old handler */
             break;
 
-        case INSTR_set_handler: /* Set up a handler */
+        case INSTR_set_handler_new: /* Set up a handler */
+            *(--sp) = PolyWord::FromCodePtr(pc + *pc + 1); /* Address of handler */
+            taskData->stack->stack()->p_hr = sp;
+            pc += 1;
+            break;
+
+        case INSTR_set_handler_old: /* Set up a handler */
+            // Legacy version.  The handler pushes an exception id.
             *(--sp) = PolyWord::FromCodePtr(pc + *pc + 1); /* Address of handler */
             taskData->stack->stack()->p_hr = sp-1; /*Point to identifier about to be pushed*/
             pc += 1;
@@ -292,11 +300,12 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
             {
                 PolyWord u = *sp++;
                 sp = taskData->stack->stack()->p_hr;
-                PolyWord *t;
                 PolyWord *endStack = taskData->stack->top;
-                while ((t = (*sp).AsStackAddr()) < sp || t > endStack) sp++;
-                taskData->stack->stack()->p_hr = t;
-                *sp = u;
+                if (*sp == TAGGED(0)) sp++; // Legacy
+                sp++; // Skip handler entry point
+                // Restore old handler
+                taskData->stack->stack()->p_hr = (*sp).AsStackAddr();
+                *sp = u; // Put back the result
                 pc += *pc + 1; /* Skip the handler */
                 break;
             }
@@ -305,8 +314,17 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
             if (*sp++ == True) { pc += 1; break; }
             /* else - false - take the jump */
 
+        case INSTR_jump_i_u: /* Indirect jump */
+            {
+                // This is always a forward jump
+                pc += *pc + 1;
+                pc += arg1 + 2;
+                break;
+            }
+
         case INSTR_jump_i: /* Indirect jump */
             {
+                // This is only for backwards compatibility
                 pc += *pc + 1;
                 /* This may jump backwards. */
                 int u = arg1;
@@ -315,7 +333,17 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
                 break;
             }
 
-        case INSTR_set_handler_i: /* Set up a handler */
+        case INSTR_set_handler_new_i: /* Set up a handler */
+            {
+                byte *u = pc + *pc + 1;
+                *(--sp) = /* Address of handler */
+                    PolyWord::FromCodePtr(u + u[0] + u[1]*256 + 2);
+                taskData->stack->stack()->p_hr = sp;
+                pc += 1;
+                break;
+            }
+
+        case INSTR_set_handler_old_i: /* Set up a handler */
             {
                 byte *u = pc + *pc + 1;
                 *(--sp) = /* Address of handler */
@@ -561,18 +589,23 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
                                 else *sp = Zero;
                             }
                             else
+                            // Strictly speaking, C does not require that this uses
+                            // arithmetic shifting so we really ought to set the
+                            // high-order bits explicitly.
                                 *sp = TAGGED(UNTAGGED(*sp) >> UNTAGGED(u));
                             break;
                         }
 
                     case POLY_SYS_load_byte:
+                    case POLY_SYS_load_byte_immut:
                         {
                             POLYUNSIGNED u = UNTAGGED(*sp++);
                             *sp = TAGGED((*sp).AsCodePtr()[u]);
                             break; 
                         }
 
-                    case POLY_SYS_load_word: 
+                    case POLY_SYS_load_word:
+                    case POLY_SYS_load_word_immut:
                         {
                             POLYUNSIGNED u = UNTAGGED(*sp++);
                             *sp = (*sp).AsObjPtr()->Get(u);
@@ -620,7 +653,7 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
                         break;
 
                     case POLY_SYS_exception_trace:
-                        u = *sp; /* Procedure to call. */
+                        u = *sp; /* Function to call. */
                         *(--sp) = PolyWord::FromCodePtr(pc); /* Push a return address. */
                         *(--sp) = PolyWord::FromStackAddr(taskData->stack->stack()->p_hr); /* Push old handler */
                         *(--sp) = SPECIAL_PC_TRACE_EX; /* Marks exception trace. */
@@ -631,6 +664,18 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
                         *(--sp) = u; /* Push the procedure. */
                         goto CALL_CLOSURE;
 
+                    case POLY_SYS_exception_trace_fn:
+                        u = *sp; /* Function to call. */
+                        *(--sp) = PolyWord::FromCodePtr(pc); /* Push a return address. */
+                        *(--sp) = PolyWord::FromStackAddr(taskData->stack->stack()->p_hr); /* Push old handler */
+                        *(--sp) = SPECIAL_PC_TRACE_EX_FN; /* Marks exception trace. */
+                        *(--sp) = Zero; /* Catch everything. */
+                        taskData->stack->stack()->p_hr = sp; /* Handler is here. */
+                        pc = (SPECIAL_PC_TRACE_EX).AsCodePtr(); /* Special return address. */
+                        *(--sp) = Zero; /* Unit argument to the function. */
+                        *(--sp) = u; /* Push the procedure. */
+                        goto CALL_CLOSURE;
+
                     case POLY_SYS_is_big_endian: {
                         union { unsigned long wrd; char chrs[sizeof(unsigned long)]; } endian;
                         endian.wrd = 1;
@@ -810,17 +855,8 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
                 taskData->stack->stack()->p_sp = sp; /* Save this in case of trace. */
                 PolyWord *t = taskData->stack->stack()->p_hr;  /* First handler */
                 PolyWord *endStack = taskData->stack->top;
-                /* Handlers consist of one or more pairs of identifier and
-                   code address, followed by the address of the next handler. */
-                while (*t != Zero && *t != exId) {
-                    /* Loop until we find an ELSE handler or one that matches */
-                    t += 2; /* Go on to next. */
-                    /* If it points into stack it must be a pointer to the next
-                       handler. */
-                    if ((*t).AsStackAddr() > t && (*t).AsStackAddr() < endStack)
-                        t = (*t).AsStackAddr();
-                }
-                t++; /* Skip over the identifier to point at the code address. */
+                // The legacy version pushes an identifier which is always zero.
+                if (*t == Zero) t++;
                 if (*t == SPECIAL_PC_TRACE_EX)
                 { /* Trace this exception. */ 
                     *sp = PolyWord::FromCodePtr(pc); /* So that this proc. will be included. */
@@ -834,7 +870,25 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
                     catch (IOException) {
                     }
                     taskData->saveVec.reset(marker);
-                    // This will have reraised the xception by calling SetException
+                    // This will have reraised the exception by calling SetException
+                    goto RESTART;
+                }
+                else if (*t == SPECIAL_PC_TRACE_EX_FN)
+                {
+                    // New exception trace
+                    *sp = PolyWord::FromCodePtr(pc); /* So that this function will be included. */
+                    // exceptionToTraceException expects the new format of exception handling that is
+                    // used in the X86 code-generator.  That does not push an exception id.
+                    taskData->stack->stack()->p_hr = t;
+                    Handle marker = taskData->saveVec.mark();
+                    try {
+                        exceptionToTraceException(taskData,
+                            taskData->saveVec.push(taskData->stack->stack()->p_reg[0]));
+                    }
+                    catch (IOException) {
+                    }
+                    taskData->saveVec.reset(marker);
+                    // This will have reraised the exception by calling SetException
                     goto RESTART;
                 }
                 else if (*t == SPECIAL_PC_END_THREAD)
@@ -983,6 +1037,9 @@ int Interpreter::SwitchToPoly(TaskData *taskData)
 
         case INSTR_jump_back: pc -= *pc + 1; break;
 
+        case INSTR_jump_back16:
+            pc -= arg1 + 1; break;
+
         case INSTR_lock:
             {
                 PolyObject *obj = (*sp).AsObjPtr();
@@ -1338,13 +1395,12 @@ void Interpreter::InitInterfaceVector(void)
     add_word_to_io_area(POLY_SYS_get_length, TAGGED(POLY_SYS_get_length));
     add_word_to_io_area(POLY_SYS_get_flags, TAGGED(POLY_SYS_get_flags));
     add_word_to_io_area(POLY_SYS_str_compare, TAGGED(POLY_SYS_str_compare));
-    add_word_to_io_area(POLY_SYS_teststreq, TAGGED(POLY_SYS_teststreq));
-    add_word_to_io_area(POLY_SYS_teststrneq, TAGGED(POLY_SYS_teststrneq));
     add_word_to_io_area(POLY_SYS_teststrgtr, TAGGED(POLY_SYS_teststrgtr));
     add_word_to_io_area(POLY_SYS_teststrlss, TAGGED(POLY_SYS_teststrlss));
     add_word_to_io_area(POLY_SYS_teststrgeq, TAGGED(POLY_SYS_teststrgeq));
     add_word_to_io_area(POLY_SYS_teststrleq, TAGGED(POLY_SYS_teststrleq));
     add_word_to_io_area(POLY_SYS_exception_trace, TAGGED(POLY_SYS_exception_trace));
+    add_word_to_io_area(POLY_SYS_exception_trace_fn, TAGGED(POLY_SYS_exception_trace_fn));
     add_word_to_io_area(POLY_SYS_lockseg, TAGGED(POLY_SYS_lockseg));
     add_word_to_io_area(POLY_SYS_profiler, TAGGED(POLY_SYS_profiler));
     add_word_to_io_area(POLY_SYS_is_short, TAGGED(POLY_SYS_is_short));
@@ -1387,6 +1443,7 @@ void Interpreter::InitInterfaceVector(void)
     add_word_to_io_area(POLY_SYS_exp_real, TAGGED(POLY_SYS_exp_real));
     add_word_to_io_area(POLY_SYS_ln_real, TAGGED(POLY_SYS_ln_real));
     add_word_to_io_area(POLY_SYS_io_operation, TAGGED(POLY_SYS_io_operation));
+    add_word_to_io_area(POLY_SYS_atomic_reset, TAGGED(POLY_SYS_atomic_reset));
     add_word_to_io_area(POLY_SYS_atomic_incr, TAGGED(POLY_SYS_atomic_incr));
     add_word_to_io_area(POLY_SYS_atomic_decr, TAGGED(POLY_SYS_atomic_decr));
     add_word_to_io_area(POLY_SYS_thread_self, TAGGED(POLY_SYS_thread_self));
@@ -1421,7 +1478,9 @@ void Interpreter::InitInterfaceVector(void)
     add_word_to_io_area(POLY_SYS_word_lss, TAGGED(POLY_SYS_word_lss));
     add_word_to_io_area(POLY_SYS_word_eq, TAGGED(POLY_SYS_word_eq));
     add_word_to_io_area(POLY_SYS_load_byte, TAGGED(POLY_SYS_load_byte));
+    add_word_to_io_area(POLY_SYS_load_byte_immut, TAGGED(POLY_SYS_load_byte_immut));
     add_word_to_io_area(POLY_SYS_load_word, TAGGED(POLY_SYS_load_word));
+    add_word_to_io_area(POLY_SYS_load_word_immut, TAGGED(POLY_SYS_load_word_immut));
     add_word_to_io_area(POLY_SYS_assign_byte, TAGGED(POLY_SYS_assign_byte));
     add_word_to_io_area(POLY_SYS_assign_word, TAGGED(POLY_SYS_assign_word));
     add_word_to_io_area(POLY_SYS_timing_dispatch, TAGGED(POLY_SYS_timing_dispatch));
@@ -1442,6 +1501,28 @@ void Interpreter::InitInterfaceVector(void)
     add_word_to_io_area(POLY_SYS_set_code_constant,TAGGED(POLY_SYS_set_code_constant));
     add_word_to_io_area(POLY_SYS_move_bytes,       TAGGED(POLY_SYS_move_bytes));
     add_word_to_io_area(POLY_SYS_move_words,       TAGGED(POLY_SYS_move_words));
+    add_word_to_io_area(POLY_SYS_move_bytes_overlap,        TAGGED(POLY_SYS_move_bytes_overlap));
+    add_word_to_io_area(POLY_SYS_move_words_overlap,        TAGGED(POLY_SYS_move_words_overlap));
+    add_word_to_io_area(POLY_SYS_eq_longword,               TAGGED(POLY_SYS_eq_longword));
+    add_word_to_io_area(POLY_SYS_neq_longword,              TAGGED(POLY_SYS_neq_longword));
+    add_word_to_io_area(POLY_SYS_geq_longword,              TAGGED(POLY_SYS_geq_longword));
+    add_word_to_io_area(POLY_SYS_leq_longword,              TAGGED(POLY_SYS_leq_longword));
+    add_word_to_io_area(POLY_SYS_gt_longword,               TAGGED(POLY_SYS_gt_longword));
+    add_word_to_io_area(POLY_SYS_lt_longword,               TAGGED(POLY_SYS_lt_longword));
+    add_word_to_io_area(POLY_SYS_plus_longword,             TAGGED(POLY_SYS_plus_longword));
+    add_word_to_io_area(POLY_SYS_minus_longword,            TAGGED(POLY_SYS_minus_longword));
+    add_word_to_io_area(POLY_SYS_mul_longword,              TAGGED(POLY_SYS_mul_longword));
+    add_word_to_io_area(POLY_SYS_div_longword,              TAGGED(POLY_SYS_div_longword));
+    add_word_to_io_area(POLY_SYS_mod_longword,              TAGGED(POLY_SYS_mod_longword));
+    add_word_to_io_area(POLY_SYS_andb_longword,             TAGGED(POLY_SYS_andb_longword));
+    add_word_to_io_area(POLY_SYS_orb_longword,              TAGGED(POLY_SYS_orb_longword));
+    add_word_to_io_area(POLY_SYS_xorb_longword,             TAGGED(POLY_SYS_xorb_longword));
+    add_word_to_io_area(POLY_SYS_shift_left_longword,       TAGGED(POLY_SYS_shift_left_longword));
+    add_word_to_io_area(POLY_SYS_shift_right_longword,      TAGGED(POLY_SYS_shift_right_longword));
+    add_word_to_io_area(POLY_SYS_shift_right_arith_longword,TAGGED(POLY_SYS_shift_right_arith_longword));
+    add_word_to_io_area(POLY_SYS_longword_to_tagged,        TAGGED(POLY_SYS_longword_to_tagged));
+    add_word_to_io_area(POLY_SYS_signed_to_longword,        TAGGED(POLY_SYS_signed_to_longword));
+    add_word_to_io_area(POLY_SYS_unsigned_to_longword,      TAGGED(POLY_SYS_unsigned_to_longword));
 
     add_word_to_io_area(POLY_SYS_io_dispatch, TAGGED(POLY_SYS_io_dispatch));
     add_word_to_io_area(POLY_SYS_network, TAGGED(POLY_SYS_network));
@@ -1449,6 +1530,46 @@ void Interpreter::InitInterfaceVector(void)
     add_word_to_io_area(POLY_SYS_signal_handler, TAGGED(POLY_SYS_signal_handler));
 }
 
+// As far as possible we want locking and unlocking an ML mutex to be fast so
+// we try to implement the code in the assembly code using appropriate
+// interlocked instructions.  That does mean that if we need to lock and
+// unlock an ML mutex in this code we have to use the same, machine-dependent,
+// code to do it.  These are defaults that are used where there is no
+// machine-specific code.
+
+// Increment the value contained in the first word of the mutex.
+// On most platforms this code will be done with a piece of assembly code.
+static PLock mutexLock;
+
+Handle Interpreter::AtomicIncrement(TaskData *taskData, Handle mutexp)
+{
+    PLocker l(&mutexLock);
+    PolyObject *p = DEREFHANDLE(mutexp);
+    // A thread can only call this once so the values will be short
+    PolyWord newValue = TAGGED(UNTAGGED(p->Get(0))+1);
+    p->Set(0, newValue);
+    return taskData->saveVec.push(newValue);
+}
+
+// Decrement the value contained in the first word of the mutex.
+Handle Interpreter::AtomicDecrement(TaskData *taskData, Handle mutexp)
+{
+    PLocker l(&mutexLock);
+    PolyObject *p = DEREFHANDLE(mutexp);
+    PolyWord newValue = TAGGED(UNTAGGED(p->Get(0))-1);
+    p->Set(0, newValue);
+    return taskData->saveVec.push(newValue);
+}
+
+// Release a mutex.  We need to lock the mutex to ensure we don't
+// reset it in the time between one of atomic operations reading
+// and writing the mutex.
+void Interpreter::AtomicReset(TaskData * /*taskData*/, Handle mutexp)
+{
+    PLocker l(&mutexLock);
+    DEREFHANDLE(mutexp)->Set(0, TAGGED(1)); // Set this to released.
+}
+
 static Interpreter interpreterObject;
 
 MachineDependent *machineDependent = &interpreterObject;
