@@ -201,6 +201,18 @@ LocalMemSpace *MemMgr::CreateAllocationSpace(POLYUNSIGNED size)
     return result;
 }
 
+// If an allocation space has a lot of data left in it after a GC, particularly 
+// a single large object we should turn it into a local area.
+void MemMgr::ConvertAllocationSpaceToLocal(LocalMemSpace *space)
+{
+    ASSERT(space->allocationSpace);
+    space->allocationSpace = false;
+    // Currently it is left as a mutable area but if the contents are all
+    // immutable e.g. a large vector it could be better to turn it into an
+    // immutable area.
+    currentAllocSpace -= space->spaceSize();
+}
+
 // Add a local memory space to the table.
 bool MemMgr::AddLocalSpace(LocalMemSpace *space)
 {
@@ -579,8 +591,12 @@ PolyWord *MemMgr::AllocHeapSpace(POLYUNSIGNED minWords, POLYUNSIGNED &maxWords,
     if (minWords > defaultSpaceSize && minWords < spaceBeforeMinorGC)
         RemoveExcessAllocation(spaceBeforeMinorGC - minWords);
 
-    if (currentAllocSpace < spaceBeforeMinorGC && minWords < spaceBeforeMinorGC - currentAllocSpace)
+    if (currentAllocSpace/* + minWords */ < spaceBeforeMinorGC)
     {
+        // i.e. the current allocation space is less than the space allowed for the minor GC
+        // but it may be that allocating this object will take us over the limit.  We allow
+        // that to happen so that we can successfully allocate very large objects even if
+        // we have a new GC very shortly.
         POLYUNSIGNED spaceSize = defaultSpaceSize;
         if (minWords > spaceSize) spaceSize = minWords; // If we really want a large space.
         LocalMemSpace *space = CreateAllocationSpace(spaceSize);
@@ -821,9 +837,11 @@ bool MemMgr::GrowOrShrinkStack(StackSpace *space, POLYUNSIGNED newSize)
     if (debugOptions & DEBUG_MEMMGR)
         Log("MMGR: Size of stack %p changed from %lu to %lu at %p\n", space, space->spaceSize(), newSize, newSpace);
     RemoveTree(space); // Remove it BEFORE freeing the space - another thread may allocate it
-    osMemoryManager->Free(space->bottom, (char*)space->top - (char*)space->bottom);
-    space->bottom = newSpace;
+    PolyWord *oldBottom = space->bottom;
+    size_t oldSize = (char*)space->top - (char*)space->bottom;
+    space->bottom = newSpace; // Switch this before freeing - We could get a profile trap during the free
     space->top = newSpace+newSize;
+    osMemoryManager->Free(oldBottom, oldSize);
     return true;
 }
 
@@ -967,6 +985,17 @@ void MemMgr::RemoveTreeRange(SpaceTree **tt, MemSpace *space, uintptr_t startS,
     *tt = 0;
 }
 
+POLYUNSIGNED MemMgr::AllocatedInAlloc()
+{
+    POLYUNSIGNED inAlloc = 0;
+    for (unsigned i = 0; i < nlSpaces; i++)
+    {
+        LocalMemSpace *sp = lSpaces[i];
+        if (sp->allocationSpace) inAlloc += sp->allocatedSpace();
+    }
+    return inAlloc;
+}
+
 // Report heap sizes and occupancy before and after GC
 void MemMgr::ReportHeapSizes(const char *phase)
 {
