@@ -94,6 +94,7 @@ time_t exportTimeStamp;
 enum {
     OPT_HEAPMIN,
     OPT_HEAPMAX,
+    OPT_HEAPINIT,
     OPT_GCPERCENT,
     OPT_RESERVE,
     OPT_GCTHREADS,
@@ -106,7 +107,7 @@ static struct __argtab {
     unsigned argKey;
 } argTable[] =
 {
-    { "-H",             "Minimum heap size (MB)",                               OPT_HEAPMIN },
+    { "-H",             "Initial heap size (MB)",                               OPT_HEAPINIT },
     { "--minheap",      "Minimum heap size (MB)",                               OPT_HEAPMIN },
     { "--maxheap",      "Maximum heap size (MB)",                               OPT_HEAPMAX },
     { "--gcpercent",    "Target percentage time in GC (1-99)",                  OPT_GCPERCENT },
@@ -179,7 +180,7 @@ unsigned parseSize(const char *p, const char *arg)
 /* In the Windows version this is called from WinMain in Console.c */
 int polymain(int argc, char **argv, exportDescription *exports)
 {
-    unsigned minsize=0, maxsize=0, gcpercent=0;
+    unsigned minsize=0, maxsize=0, gcpercent=0, initsize=0;
     /* Get arguments. */
     memset(&userOptions, 0, sizeof(userOptions)); /* Reset it */
     userOptions.gcthreads = 0; // Default multi-threaded
@@ -230,6 +231,9 @@ int polymain(int argc, char **argv, exportDescription *exports)
                     case OPT_HEAPMAX:
                         maxsize = parseSize(p, argTable[j].argName);
                         break;
+                    case OPT_HEAPINIT:
+                        initsize = parseSize(p, argTable[j].argName);
+                        break;
                     case OPT_GCPERCENT:
                         gcpercent = strtol(p, &endp, 10);
                         if (*endp != '\0') 
@@ -293,13 +297,30 @@ int polymain(int argc, char **argv, exportDescription *exports)
     if (exports == 0 && importFileName == 0)
         Usage("Missing import file name\n");
 
+    // If the maximum is provided it must be not less than the minimum.
+    if (maxsize != 0 && maxsize < minsize)
+        Usage("Minimum heap size must not be more than maximum size\n");
+    // The initial size must be not more than the maximum
+    if (maxsize != 0 && maxsize < initsize)
+        Usage("Initial heap size must not be more than maximum size\n");
+    // The initial size must be not less than the minimum
+    if (initsize != 0 && initsize < minsize)
+        Usage("Initial heap size must not be less than minimum size\n");
+
     if (userOptions.gcthreads == 0)
-        // For the moment, at any rate, indicate that we should use as many
-        // threads as there are processors by a thread count of zero.
-        userOptions.gcthreads = NumberOfProcessors();
+    {
+        // If the gcthreads option is missing or zero the default is to try to
+        // use as many threads as there are physical processors.  The result may
+        // be zero in which case we use the number of processors.  Because memory
+        // bandwidth is a limiting factor we want to avoid muliple GC threads on
+        // hyperthreaded "processors".
+        userOptions.gcthreads = NumberOfPhysicalProcessors();
+        if (userOptions.gcthreads == 0)
+            userOptions.gcthreads = NumberOfProcessors();
+    }
 
     // Set the heap size if it has been provided otherwise use the default.
-    gHeapSizeParameters.SetHeapParameters(minsize, maxsize, gcpercent);
+    gHeapSizeParameters.SetHeapParameters(minsize, maxsize, initsize, gcpercent);
    
     // Initialise the run-time system before creating the heap.
     InitModules();
