@@ -19,10 +19,12 @@
     Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 */
-#ifdef WIN32
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#elif defined(_WIN32)
 #include "winconfig.h"
 #else
-#include "config.h"
+#error "No configuration file"
 #endif
 
 #ifdef HAVE_STDIO_H
@@ -62,21 +64,6 @@
 
 #define MAX_PROF_LEN 100 // Profile lengths between 1 and this
 
-class VisitBitmap
-{
-public:
-    VisitBitmap(PolyWord *bottom, PolyWord *top):
-        m_bitmap(top - bottom),
-        m_bottom(bottom), m_top(top) {}
-
-    bool AlreadyVisited(PolyObject *p)   { return m_bitmap.TestBit((PolyWord*)p - m_bottom); }
-    void SetVisited(PolyObject *p)       { m_bitmap.SetBit((PolyWord*)p - m_bottom); }
-
-    Bitmap     m_bitmap;
-    PolyWord  *m_bottom;
-    PolyWord  *m_top;
-};
-
 class ProcessVisitAddresses: public ScanAddress
 {
 public:
@@ -92,7 +79,7 @@ public:
     void ShowCode(PolyObject *start);
     void ShowWords(PolyObject *start);
 
-    int      total_length;
+    POLYUNSIGNED total_length;
     bool     show_size;
     void     *io_bottom;
     void     *io_top;
@@ -125,8 +112,7 @@ ProcessVisitAddresses::ProcessVisitAddresses(bool show)
     for (j = 0; j < gMem.nlSpaces; j++)
     {
         LocalMemSpace *space = gMem.lSpaces[j];
-        // Local areas only have objects from the allocation pointer to the top.
-        bitmaps[bm++] = new VisitBitmap(space->pointer, space->top);
+        bitmaps[bm++] = new VisitBitmap(space->bottom, space->top);
     }
     ASSERT(bm == nBitmaps);
 
@@ -154,7 +140,7 @@ VisitBitmap *ProcessVisitAddresses::FindBitmap(PolyWord p)
     for (unsigned i = 0; i < nBitmaps; i++)
     {
         VisitBitmap *bm = bitmaps[i];
-        if (p.AsAddress() > bm->m_bottom && p.AsAddress() <= bm->m_top) return bm;
+        if (bm->InRange(p.AsStackAddr())) return bm;
     }
     return 0;
 }
@@ -168,7 +154,7 @@ void ProcessVisitAddresses::ShowBytes(PolyObject *start)
 
     if (start->IsMutable()) printf("MUTABLE ");
 
-    printf("BYTES:%p:%lu\n", array, bytes);
+    printf("BYTES:%p:%" POLYUFMT "\n", array, bytes);
 
     POLYUNSIGNED i, n;
     for (i = 0, n = 0; n < bytes; n++)
@@ -203,7 +189,7 @@ void ProcessVisitAddresses::ShowCode(PolyObject *start)
     else
         (void) Poly_string_to_C(string, buffer, sizeof(buffer));
 
-    printf("CODE:%p:%lu %s\n", start, length, buffer);
+    printf("CODE:%p:%" POLYUFMT " %s\n", start, length, buffer);
 
     POLYUNSIGNED i, n;
     for (i = 0, n = 0; n < length; n++)
@@ -229,7 +215,7 @@ void ProcessVisitAddresses::ShowWords(PolyObject *start)
     putc('\n', stdout);
     if (start->IsMutable()) printf("MUTABLE ");
     
-    printf("WORDS:%p:%lu\n", start, length);
+    printf("WORDS:%p:%" POLYUFMT "\n", start, length);
     
     POLYUNSIGNED i, n;
     for (i = 0, n = 0; n < length; n++)
@@ -321,10 +307,6 @@ POLYUNSIGNED ProcessVisitAddresses::ShowWord(PolyWord w)
             ShowBytes(p);
         return 0;
     }
-    else if (OBJ_IS_STACK_OBJECT(L))
-    {
-        return 0; // We shouldn't get stack objects.
-    }
     else if (OBJ_IS_CODE_OBJECT(L))
     {
         PolyWord *cp;
