@@ -2,7 +2,7 @@
     Title:      Thread functions
     Author:     David C.J. Matthews
 
-    Copyright (c) 2007,2008 David C.J. Matthews
+    Copyright (c) 2007,2008,2013 David C.J. Matthews
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -178,7 +178,7 @@ public:
     LONGLONG lastCPUTime; // Used for profiling
 #endif
 #ifdef HAVE_PTHREAD
-    pthread_t pthreadId;
+    bool threadExited;
 #endif
 #ifdef HAVE_WINDOWS_H
     HANDLE threadHandle;
@@ -205,6 +205,9 @@ public:
     virtual void ThreadPauseForIO(TaskData *taskData, Waiter *pWait);
     // Return the task data for the current thread.
     virtual TaskData *GetTaskDataForThread(void);
+    // Create a new task data object for the current thread.
+    virtual TaskData *CreateNewTaskData(Handle threadId, Handle threadFunction,
+                           Handle args, PolyWord flags);
     // ForkFromRTS.  Creates a new thread from within the RTS.
     virtual bool ForkFromRTS(TaskData *taskData, Handle proc, Handle arg);
     // Create a new thread.  The "args" argument is only used for threads
@@ -344,46 +347,6 @@ static POLYUNSIGNED ThreadAttrs(TaskData *taskData)
     return UNTAGGED_UNSIGNED(taskData->threadObject->flags);
 }
 
-// As far as possible we want locking and unlocking an ML mutex to be fast so
-// we try to implement the code in the assembly code using appropriate
-// interlocked instructions.  That does mean that if we need to lock and
-// unlock an ML mutex in this code we have to use the same, machine-dependent,
-// code to do it.  These are defaults that are used where there is no
-// machine-specific code.
-
-// Increment the value contained in the first word of the mutex.
-// On most platforms this code will be done with a piece of assembly code.
-PLock mutexLock;
-
-Handle MachineDependent::AtomicIncrement(TaskData *taskData, Handle mutexp)
-{
-    mutexLock.Lock();
-    PolyObject *p = DEREFHANDLE(mutexp);
-    // A thread can only call this once so the values will be short
-    PolyWord newValue = TAGGED(UNTAGGED(p->Get(0))+1);
-    p->Set(0, newValue);
-    mutexLock.Unlock();
-    return SAVE(newValue);
-}
-
-// Decrement the value contained in the first word of the mutex.
-Handle MachineDependent::AtomicDecrement(TaskData *taskData, Handle mutexp)
-{
-    mutexLock.Lock();
-    PolyObject *p = DEREFHANDLE(mutexp);
-    PolyWord newValue = TAGGED(UNTAGGED(p->Get(0))-1);
-    p->Set(0, newValue);
-    mutexLock.Unlock();
-    return SAVE(newValue);
-}
-
-// Release a mutex and if necessary apply a write barrier.
-// This is the default code for platforms that don't need a barrier.
-void MachineDependent::SetToReleased(TaskData * /*taskData*/, Handle mutexp)
-{
-    DEREFHANDLE(mutexp)->Set(0, TAGGED(1)); // Set this to released.
-}
-
 // Called from interface vector.  Generally the assembly code will be
 // used instead of this.
 Handle AtomicIncrement(TaskData *taskData, Handle mutexp)
@@ -398,6 +361,12 @@ Handle AtomicDecrement(TaskData *taskData, Handle mutexp)
     return machineDependent->AtomicDecrement(taskData, mutexp);
 }
 
+Handle AtomicReset(TaskData *taskData, Handle mutexp)
+{
+    machineDependent->AtomicReset(taskData, mutexp);
+    return SAVE(TAGGED(0)); // Push the unit result
+}
+
 // Return the thread object for the current thread.
 // On most platforms this will be done with a piece of assembly code.
 Handle ThreadSelf(TaskData *taskData)
@@ -534,7 +503,7 @@ Handle Processes::ThreadDispatch(TaskData *taskData, Handle args, Handle code)
             Handle decrResult = machineDependent->AtomicIncrement(taskData, mutexH);
             if (UNTAGGED(decrResult->Word()) != 1)
             {
-                machineDependent->SetToReleased(taskData, mutexH);
+                machineDependent->AtomicReset(taskData, mutexH);
                 // The mutex was locked so we have to release any waiters.
                 // Unlock any waiters.
                 for (unsigned i = 0; i < taskArraySize; i++)
@@ -641,7 +610,7 @@ Handle Processes::ThreadDispatch(TaskData *taskData, Handle args, Handle code)
 
     case 13: // Return the number of processors.
         // Returns 1 if there is any problem.
-        return Make_unsigned(taskData, NumberOfProcessors());
+        return Make_arbitrary_precision(taskData, NumberOfProcessors());
 
     default:
         {
@@ -681,12 +650,12 @@ ProcessTaskData::ProcessTaskData(): requests(kRequestNone), blockMutex(0), inMLH
 #ifdef HAVE_WINDOWS_H
     lastCPUTime = 0;
 #endif
-#ifdef HAVE_PTHREAD
-    pthreadId = 0;
-#endif
 #ifdef HAVE_WINDOWS_H
     threadHandle = 0;
 #endif
+#ifdef HAVE_PTHREAD
+    threadExited = false;
+#endif
 }
 
 ProcessTaskData::~ProcessTaskData()
@@ -755,6 +724,17 @@ void Processes::ThreadExit(TaskData *taskData)
     if (debugOptions & DEBUG_THREADS)
         Log("THREAD: Thread %p exiting\n", taskData);
 
+#ifdef HAVE_PTHREAD
+    // Block any profile interrupt from now on.  We're deleting the ML stack for this thread.
+    sigset_t block_sigs;
+    sigemptyset(&block_sigs);
+    sigaddset(&block_sigs, SIGVTALRM);
+    pthread_sigmask(SIG_BLOCK, &block_sigs, NULL);
+    // Remove the thread-specific data since it's no
+    // longer valid.
+    pthread_setspecific(tlsId, 0);
+#endif
+
     globalStats.decCount(PSC_THREADS);
 
     if (singleThreaded) finish(0);
@@ -1100,6 +1080,83 @@ TaskData *Processes::GetTaskDataForThread(void)
 #endif
 }
 
+// Called to create a task data object in the current thread.
+// This is currently only used if a thread created in foreign code calls
+// a callback.
+TaskData *Processes::CreateNewTaskData(Handle threadId, Handle threadFunction,
+                           Handle args, PolyWord flags)
+{
+    ProcessTaskData *taskData = new ProcessTaskData;
+    taskData->mdTaskData = machineDependent->CreateTaskData();
+#if defined(HAVE_WINDOWS_H)
+    HANDLE thisProcess = GetCurrentProcess();
+    DuplicateHandle(thisProcess, GetCurrentThread(), thisProcess, 
+        &(taskData->threadHandle), THREAD_ALL_ACCESS, FALSE, 0);
+#endif
+    unsigned thrdIndex;
+
+    {
+        PLocker lock(&schedLock);
+        // See if there's a spare entry in the array.
+        for (thrdIndex = 0;
+                thrdIndex < taskArraySize && taskArray[thrdIndex] != 0;
+                thrdIndex++);
+
+        if (thrdIndex == taskArraySize) // Need to expand the array
+        {
+            ProcessTaskData **newArray =
+                (ProcessTaskData **)realloc(taskArray, sizeof(ProcessTaskData *)*(taskArraySize+1));
+            if (newArray)
+            {
+                taskArray = newArray;
+                taskArraySize++;
+            }
+            else
+            {
+                delete(taskData);
+                schedLock.Unlock();
+                throw MemoryException();
+            }
+        }
+        // Add into the new entry
+        taskArray[thrdIndex] = taskData;
+    }
+
+    taskData->stack = gMem.NewStackSpace(machineDependent->InitialStackSize());
+    if (taskData->stack == 0)
+    {
+        delete(taskData);
+        throw MemoryException();
+    }
+
+    machineDependent->InitStackFrame(taskData, taskData->stack, threadFunction, args);
+
+    ThreadUseMLMemory(taskData);
+
+    // If the forking thread has created an ML thread object use that
+    // otherwise create a new one in the current context.
+    if (threadId != 0)
+        taskData->threadObject = (ThreadObject*)threadId->WordP();
+    else
+    {
+        taskData->threadObject = (ThreadObject*)alloc(taskData, 4, F_MUTABLE_BIT);
+        taskData->threadObject->index = TAGGED(thrdIndex); // Set to the index
+        taskData->threadObject->flags = flags != TAGGED(0) ? TAGGED(PFLAG_SYNCH): flags;
+        taskData->threadObject->threadLocal = TAGGED(0); // Empty thread-local store
+        taskData->threadObject->requestCopy = TAGGED(0); // Cleared interrupt state
+    }
+
+#ifdef HAVE_PTHREAD
+    initThreadSignals(taskData);
+    pthread_setspecific(tlsId, taskData);
+#elif defined(HAVE_WINDOWS_H)
+    TlsSetValue(tlsId, taskData);
+#endif
+    globalStats.incCount(PSC_THREADS);
+
+    return taskData;
+}
+
 // This function is run when a new thread has been forked.  The
 // parameter is the taskData value for the new thread.  This function
 // is also called directly for the main thread.
@@ -1191,9 +1248,7 @@ void Processes::BeginRootThread(PolyObject *rootFunction)
         taskData->threadObject->flags = TAGGED(PFLAG_BROADCAST|PFLAG_ASYNCH); // Flags
         taskData->threadObject->threadLocal = TAGGED(0); // Empty thread-local store
         taskData->threadObject->requestCopy = TAGGED(0); // Cleared interrupt state
-#ifdef HAVE_PTHREAD
-        taskData->pthreadId = pthread_self();
-#elif defined(HAVE_WINDOWS_H)
+#if defined(HAVE_WINDOWS_H)
         taskData->threadHandle = mainThreadHandle;
 #endif
         taskArray[0] = taskData;
@@ -1212,7 +1267,6 @@ void Processes::BeginRootThread(PolyObject *rootFunction)
             interrupt_exn =
                 DEREFEXNHANDLE(make_exn(taskData, EXC_interrupt, taskData->saveVec.push(TAGGED(0))));
 
-
         if (singleThreaded)
         {
             // If we don't have threading enter the code as if this were a new thread.
@@ -1228,13 +1282,15 @@ void Processes::BeginRootThread(PolyObject *rootFunction)
         pthread_attr_t attrs;
         pthread_attr_init(&attrs);
         pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);
-        if (pthread_create(&taskData->pthreadId, &attrs, NewThreadFunction, taskData) != 0)
+        // N.B.  Because we create the thread detached the thread ID will be deleted
+        // by the thread itself so may be invalid at any time, unlike the Windows handle.
+        pthread_t pthreadId;
+        if (pthread_create(&pthreadId, &attrs, NewThreadFunction, taskData) != 0)
             errorCode = errno;
         pthread_attr_destroy(&attrs);
 #elif defined(HAVE_WINDOWS_H)
-        DWORD dwThrdId; // Have to provide this although we don't use it.
         taskData->threadHandle =
-            CreateThread(NULL, 0, NewThreadFunction, taskData, 0, &dwThrdId);
+            CreateThread(NULL, 0, NewThreadFunction, taskData, 0, NULL);
         if (taskData->threadHandle == NULL) errorCode = 0-GetLastError();
 #endif
         if (errorCode != 0)
@@ -1267,13 +1323,30 @@ void Processes::BeginRootThread(PolyObject *rootFunction)
             {
                 if (p == sigTask) signalThreadRunning = true;
                 else noUserThreads = false;
-            }
-            if (p && p->inMLHeap)
-            {
-                allStopped = false;
-                // It must be running - interrupt it if we are waiting.
-                if (threadRequest != 0)
-                    machineDependent->InterruptCode(p);
+
+                if (p->inMLHeap)
+                {
+                    allStopped = false;
+                    // It must be running - interrupt it if we are waiting.
+                    if (threadRequest != 0)
+                        machineDependent->InterruptCode(p);
+                }
+                else
+                {
+                    // Has the thread terminated in foreign code?
+                    bool thread_killed = false;
+#ifdef HAVE_PTHREAD
+                    thread_killed = p->threadExited;
+#elif defined(HAVE_WINDOWS_H)
+                    thread_killed = WaitForSingleObject(p->threadHandle, 0) == WAIT_OBJECT_0;
+#endif
+                    if (thread_killed)
+                    {
+                        delete(p);
+                        taskArray[i] = 0;
+                        globalStats.decCount(PSC_THREADS);
+                    }
+                }
             }
         }
         if (noUserThreads)
@@ -1304,10 +1377,12 @@ void Processes::BeginRootThread(PolyObject *rootFunction)
             for (unsigned i = 0; i < taskArraySize; i++)
             {
                 ProcessTaskData *taskData = taskArray[i];
-                if (taskData)
+                if (taskData && taskData->requests != kRequestKill)
                     MakeRequest(taskData, kRequestKill);
             }
-            exitRequest = false; // Don't need to repeat this.
+            // Leave exitRequest set so that if we're in the process of
+            // creating a new thread we will request it to stop when the
+            // taskData object has been added to the table.
         }
 
         // Now release schedLock and wait for a thread
@@ -1434,12 +1509,12 @@ Handle Processes::ForkThread(ProcessTaskData *taskData, Handle threadFunction,
         pthread_attr_t attrs;
         pthread_attr_init(&attrs);
         pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);
-        success = pthread_create(&newTaskData->pthreadId, &attrs, NewThreadFunction, newTaskData) == 0;
+        pthread_t pthreadId;
+        success = pthread_create(&pthreadId, &attrs, NewThreadFunction, newTaskData) == 0;
         pthread_attr_destroy(&attrs);
 #elif defined(HAVE_WINDOWS_H)
-        DWORD dwThrdId; // Have to provide this although we don't use it.
         newTaskData->threadHandle =
-            CreateThread(NULL, 0, NewThreadFunction, newTaskData, 0, &dwThrdId);
+            CreateThread(NULL, 0, NewThreadFunction, newTaskData, 0, NULL);
         success = newTaskData->threadHandle != NULL;
 #endif
         if (success)
@@ -1843,6 +1918,16 @@ void Processes::SignalArrived(void)
         sigTask->threadLock.Signal();
 }
 
+#ifdef HAVE_PTHREAD
+// This is called when the thread exits in foreign code and
+// ThreadExit has not been called.  Normally the thread-specific
+// data is cleared.
+static void threaddata_destructor(void *p)
+{
+    ProcessTaskData *pt = (ProcessTaskData *)p;
+    pt->threadExited = true;
+}
+#endif
 
 void Processes::Init(void)
 {
@@ -1852,7 +1937,7 @@ void Processes::Init(void)
 #endif
 
 #ifdef HAVE_PTHREAD
-    pthread_key_create(&tlsId, NULL);
+    pthread_key_create(&tlsId, threaddata_destructor);
 #elif defined(HAVE_WINDOWS_H)
     tlsId = TlsAlloc();
 #else
@@ -1971,7 +2056,7 @@ void ProcessTaskData::GarbageCollect(ScanAddress *process)
     process->ScanRuntimeWord(&foreignStack);
 }
 
-// Return the number of processors.  Used when configuring multi-threaded GC.
+// Return the number of processors.
 extern unsigned NumberOfProcessors(void)
 {
 #if (defined(_WIN32) && ! defined(__CYGWIN__))
@@ -1997,3 +2082,99 @@ extern unsigned NumberOfProcessors(void)
         return 1;
 #endif
 }
+
+// Return the number of physical processors.  If hyperthreading is
+// enabled this returns less than NumberOfProcessors.  Returns zero if
+// it cannot be determined.
+#if (defined(_WIN32) && defined(HAVE_SYSTEM_LOGICAL_PROCESSOR_INFORMATION))
+typedef BOOL (WINAPI *GETP)(SYSTEM_LOGICAL_PROCESSOR_INFORMATION*, PDWORD);
+
+// Windows - use GetLogicalProcessorInformation if it's available.
+static unsigned WinNumPhysicalProcessors(void)
+{
+    GETP getProcInfo = (GETP) GetProcAddress(GetModuleHandle("kernel32"), "GetLogicalProcessorInformation");
+    if (getProcInfo == 0) return 0;
+
+    // It's there - use it.
+    SYSTEM_LOGICAL_PROCESSOR_INFORMATION *buff = 0;
+    DWORD space = 0;
+    while (getProcInfo(buff, &space) == FALSE)
+    {
+        if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
+        {
+            free(buff);
+            return 0;
+        }
+        free(buff);
+        buff = (PSYSTEM_LOGICAL_PROCESSOR_INFORMATION)malloc(space);
+        if (buff == 0) return 0;
+    }
+    // Calculate the number of full entries in case it's truncated.
+    unsigned nItems = space / sizeof(SYSTEM_LOGICAL_PROCESSOR_INFORMATION);
+    unsigned numProcs = 0;
+    for (unsigned i = 0; i < nItems; i++)
+    {
+        if (buff[i].Relationship == RelationProcessorCore)
+            numProcs++;
+    }
+    free(buff);
+    return numProcs;
+}
+#endif
+
+// Read and parse /proc/cpuinfo
+static unsigned LinuxNumPhysicalProcessors(void)
+{
+    // Find out the total.  This should be the maximum.
+    unsigned nProcs = NumberOfProcessors();
+    // If there's only one we don't need to check further.
+    if (nProcs <= 1) return nProcs;
+    long *cpus = (long*)malloc(nProcs * sizeof(long));
+    if (cpus == 0) return 0;
+    memset(cpus, 0, nProcs * sizeof(long));
+
+    FILE *cpuInfo = fopen("/proc/cpuinfo", "r");
+    if (cpuInfo == NULL) { free(cpus); return 0; }
+
+    char line[40];
+    unsigned count = 0;
+    while (fgets(line, sizeof(line), cpuInfo) != NULL)
+    {
+        if (strncmp(line, "core id\t\t:", 10) == 0)
+        {
+            long n = strtol(line+10, NULL, 10);
+            unsigned i = 0;
+            // Skip this id if we've seen it already
+            while (i < count && cpus[i] != n) i++;
+            if (i == count) cpus[count++] = n;
+        }
+        if (strchr(line, '\n') == 0)
+        {
+            int ch;
+            do { ch = getc(cpuInfo); } while (ch != '\n' && ch != EOF);
+        }
+    }
+
+    fclose(cpuInfo);
+    free(cpus);
+    return count;
+}
+
+extern unsigned NumberOfPhysicalProcessors(void)
+{
+    unsigned numProcs = 0;
+#if (defined(_WIN32) && defined(HAVE_SYSTEM_LOGICAL_PROCESSOR_INFORMATION))
+    numProcs = WinNumPhysicalProcessors();
+    if (numProcs != 0) return numProcs;
+#endif
+#if (defined(HAVE_SYSCTLBYNAME) && defined(HAVE_SYS_SYSCTL_H))
+    // Mac OS X
+    int nCores;
+    size_t len = sizeof(nCores);
+    if (sysctlbyname("hw.physicalcpu", &nCores, &len, NULL, 0) == 0)
+        return (unsigned)nCores;
+#endif
+    numProcs = LinuxNumPhysicalProcessors();
+    if (numProcs != 0) return numProcs;
+    return numProcs;
+}
