@@ -3,7 +3,7 @@
 
     Copyright (c) 2000-7
         Cambridge University Technical Services Limited
-    Further development Copyright David C.J. Matthews 2007-11
+    Further development Copyright David C.J. Matthews 2007-12
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -42,39 +42,53 @@
 #define POLY_SYS_get_length          15
 #define POLY_SYS_get_flags           17
 #define POLY_SYS_str_compare         23 /* DCJM 21/3/06 */
-#define POLY_SYS_teststreq           24
-#define POLY_SYS_teststrneq          25
 #define POLY_SYS_teststrgtr          26
 #define POLY_SYS_teststrlss          27
 #define POLY_SYS_teststrgeq          28
 #define POLY_SYS_teststrleq          29
 #define POLY_SYS_exception_trace     30
 #define POLY_SYS_give_ex_trace       31 /* Called from exception unwind code. */
+#define POLY_SYS_exception_trace_fn  32
+#define POLY_SYS_give_ex_trace_fn    33 // Called to produce the exception trace
 #define POLY_SYS_lockseg             47
-#define POLY_SYS_emptystring         48
-#define POLY_SYS_nullvector          49
+#define POLY_SYS_emptystring         48 // A value not a function
+#define POLY_SYS_nullvector          49 // A value not a function
 #define POLY_SYS_network             51 /* DCJM 22/5/00 */
 #define POLY_SYS_os_specific         52 /* DCJM 22/5/00 */
+#define POLY_SYS_eq_longword                    53
+#define POLY_SYS_neq_longword                   54
+#define POLY_SYS_geq_longword                   55
+#define POLY_SYS_leq_longword                   56
+#define POLY_SYS_gt_longword                    57
+#define POLY_SYS_lt_longword                    58
 #define POLY_SYS_io_dispatch         61 /* DCJM 8/5/00 */
 #define POLY_SYS_signal_handler      62 /* DCJM 18/7/00 */
+#define POLY_SYS_atomic_reset        69 /* DCJM 18/9/12 */
 #define POLY_SYS_atomic_incr         70 /* DCJM 12/3/07 */
 #define POLY_SYS_atomic_decr         71 /* DCJM 12/3/07 */
 #define POLY_SYS_thread_self         72 /* DCJM 12/3/07 */
 #define POLY_SYS_thread_dispatch     73 /* DCJM 12/3/07 */
-
-//#define POLY_SYS_fork_process        82
-//#define POLY_SYS_choice_process      83
+#define POLY_SYS_plus_longword                  74
+#define POLY_SYS_minus_longword                 75
+#define POLY_SYS_mul_longword                   76
+#define POLY_SYS_div_longword                   77
+#define POLY_SYS_mod_longword                   78
+#define POLY_SYS_andb_longword                  79
+#define POLY_SYS_orb_longword                   80
+#define POLY_SYS_xorb_longword                  81
 #define POLY_SYS_kill_self           84
-//#define POLY_SYS_int_process         85
-//#define POLY_SYS_send_on_channel     86
-//#define POLY_SYS_receive_on_channel  87
+#define POLY_SYS_shift_left_longword            85
+#define POLY_SYS_shift_right_longword           86
+#define POLY_SYS_shift_right_arith_longword     87
 #define POLY_SYS_profiler            88
+#define POLY_SYS_longword_to_tagged             89
+#define POLY_SYS_signed_to_longword             90
+#define POLY_SYS_unsigned_to_longword           91
 #define POLY_SYS_full_gc             92   /* MJC 18/03/91 */
 #define POLY_SYS_stack_trace         93   /* MJC 18/03/91 */
 #define POLY_SYS_timing_dispatch     94   /* DCJM 10/4/00 */
 #define POLY_SYS_objsize             99   /* MJC 27/04/88 */
 #define POLY_SYS_showsize            100  /* MJC 09/03/89 */
-//#define POLY_SYS_interrupt_console_processes 103  /* MJC 01/08/90 */
 #define POLY_SYS_quotrem             104  /* DCJM 05/03/10 */
 #define POLY_SYS_is_short            105
 #define POLY_SYS_aplus               106
@@ -119,11 +133,13 @@
 #define POLY_SYS_poly_specific       153  /* DCJM 17/6/06 */
 #define POLY_SYS_bytevec_eq          154  /* DCJM 13/1/10 */
 #define POLY_SYS_io_operation        189
+#define POLY_SYS_move_words_overlap  193
 #define POLY_SYS_set_code_constant   194  /* DCJM 2/1/01 */
 #define POLY_SYS_move_words          195  /* DCJM 9/10/99 */
 #define POLY_SYS_shift_right_arith_word  196  /* DCJM 9/10/99 */
 #define POLY_SYS_int_to_word         197  /* DCJM 9/10/99 */ /* Obsolete: Now replaced by POLY_SYS_get_first_long_word. */
 #define POLY_SYS_move_bytes          198  /* DCJM 9/10/99 */
+#define POLY_SYS_move_bytes_overlap  199
 #define POLY_SYS_code_flags          200  /* Previously POLY_SYS_set_flags */
 #define POLY_SYS_shrink_stack        201  /* SPF  9/12/96 */
 #define POLY_SYS_stderr              202  /* SPF 29/11/96 */
@@ -144,6 +160,8 @@
 #define POLY_SYS_int_leq             232
 #define POLY_SYS_int_gtr             233
 #define POLY_SYS_int_lss             234
+#define POLY_SYS_load_byte_immut     235
+#define POLY_SYS_load_word_immut     236
 #define POLY_SYS_mul_word            238  /* Reinstated DCJM 9/10/99 */
 #define POLY_SYS_plus_word           239  /* Reinstated DCJM 9/10/99 */
 #define POLY_SYS_minus_word          240  /* Reinstated DCJM 9/10/99 */
@@ -202,6 +220,8 @@
 
 #define EXC_thread      12 /* Thread.Thread. */  /* DCJM 13/3/07 */
 
+#define EXC_extrace     13 // Used for exception tracing
+
 #define EXC_foreign     23  /* nic 4/5/94 */
 
 #define EXC_Fail        103  /* DCJM 11/5/06 */
