@@ -439,6 +439,14 @@
 /* Define to 1 if you have the `sysctl' function. */
 #undef HAVE_SYSCTL
 
+/* Define to 1 if you have the `sysctlbyname' function. */
+#undef HAVE_SYSCTLBYNAME
+ 
+/* Define to 1 if the system has the type
+   `SYSTEM_LOGICAL_PROCESSOR_INFORMATION'. */
+#undef HAVE_SYSTEM_LOGICAL_PROCESSOR_INFORMATION
+#define HAVE_SYSTEM_LOGICAL_PROCESSOR_INFORMATION 1
+
 /* Define to 1 if you have the <sys/dir.h> header file, and it defines `DIR'.
    */
 #undef HAVE_SYS_DIR_H
@@ -596,9 +604,6 @@
 /* Define if the host is an X86 (64 bit) */
 /*#undef HOSTARCHITECTURE_X86_64*/
 
-/* Define if using the interpreter */
-/*#undef INTERPRETED*/
-
 /* Define to 1 if `lstat' dereferences a symlink specified with a trailing
    slash. */
 #undef LSTAT_FOLLOWS_SLASHED_SYMLINK
@@ -644,8 +649,11 @@
 /* Define to the type of arg 5 for `select'. */
 #undef SELECT_TYPE_ARG5
 
+/* The size of `int', as computed by sizeof. */
+// N.B.  This is 4 on both 32-bit and 64-bit
+#define SIZEOF_INT 4
+
 /* The size of `long', as computed by sizeof. */
-#undef SIZEOF_LONG
 // N.B.  This is 4 on both 32-bit and 64-bit
 #define SIZEOF_LONG 4
 
@@ -657,6 +665,10 @@
 #define SIZEOF_VOIDP 4
 #endif
 
+// Size of long long
+// N.B. This is 8 on both 32-bit and 64-bit
+#define SIZEOF_LONG_LONG 8
+
 /* If using the C implementation of alloca, define if you know the
    direction of stack growth for your system; otherwise it will be
    automatically deduced at runtime.
